#include "particle.H"
#include <iostream>
#include <sys/time.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#include <cstdlib>
#include "gdTiff.H"

using std::cout;
using std::cerr;
using std::endl;

bool OUTPUT = true;

double meanparts=10;
double varparts=5;
BfastVector3 sphereCenter(0.0,0.0,200);
double sphereRadius = 10.0;
double initSpeed = 15.0;
double mass = 1.0;
double radius = 1.0;
double size = 1.0;
double crest = 0.4;
double fricNum = 0.1;
std::vector<Particle> particles;
std::vector<Obstacle*> obstacles;
  
BfastVector3 randomVector () {
  BfastVector3 x;
  x[0] = (((double)rand())/((double)RAND_MAX));
  x[1] = (((double)rand())/((double)RAND_MAX));
  x[2] = (((double)rand())/((double)RAND_MAX));
  return x;
}

void myIdle(void) {
	double dt = 1.0/10.0;

	// create particles
	int nparts = (int)(meanparts + 2.0*((((double)rand())/RAND_MAX)-0.5)*varparts);
	for (int i=0; i<nparts; i++) {
		Particle p(sphereCenter + 2*(randomVector()-0.5)*sphereRadius,
							 initSpeed*2*(randomVector()-0.5),
							 0.0, randomVector(), mass, size, radius, crest, fricNum, 0.0);
		particles.push_back(p);
	}
	// move particles
	step(particles, obstacles, dt);

	glutPostRedisplay(); 
}

void myReshape(int w, int h) { 
	glViewport (0, 0, w, h); // update the viewport 
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)640/(GLfloat)480,
								 0.001f, 10000.0f);
	gluLookAt(500, 0, 100, 0, 0, 100, 0, 0, 1);
	glMatrixMode(GL_MODELVIEW); 
	glutPostRedisplay(); 
} 

void drawObjects() { 
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	for (unsigned int i=0; i<particles.size(); i++) {
		BfastVector3 color(particles[i].col * 1.0/(1.0 + 0.05*particles[i].age));
		glColor3f(color[0], color[1], color[2]);
		glPushMatrix();
		glTranslatef(particles[i].pos[0], particles[i].pos[1], particles[i].pos[2]);
		glBegin(GL_POLYGON);
		glVertex3f(0.0, -particles[i].r, -particles[i].r);
		glVertex3f(0.0, -particles[i].r, particles[i].r);
		glVertex3f(0.0, particles[i].r, particles[i].r);
		glVertex3f(0.0, particles[i].r, -particles[i].r);
		glEnd();
		glPopMatrix();
	}
} 

void myDisplay(void) { 
	static int frame = 0;

	glClearColor(0.5, 0.5, 0.5, 0.0); 
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

	drawObjects(); 
	glFlush();
	glutSwapBuffers(); // swap buffers 

	if (OUTPUT) {
		float pixels[640*480*3];
		glReadPixels(0,0,640,480,GL_RGB,GL_FLOAT, pixels);
		GdTiff tiff(640,480,GdTiff::RGB);
		for (unsigned int i=0; i<640; i++) {
			for (unsigned int j=0; j<480; j++) {
				for (unsigned int k=0; k<3; k++) {
					tiff(i,j,k) = pixels[j*640*3+i*3+k];
				}
				tiff(i,j,3) = 1.0;
			}
		}
		char fname[80];
		sprintf(fname, "frame.%06d.tif", frame);
		tiff.write(fname);
		frame++;
	}
}

int main(int argc, char *argv[]) {
//----------------------------------------------------------------------- 
// Main program - It creates the game and then passes control to glut. 
//----------------------------------------------------------------------- 
	glutInit(&argc, argv); // OpenGL initializations 
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);// double buffering and RGB 
	glutInitWindowSize(640, 480); // create a 400x400 window 
	glutInitWindowPosition(0, 0); // ...in the upper left 
	glutCreateWindow(argv[0]); // create the window 
	glutDisplayFunc(myDisplay); // setup callbacks 
	glutReshapeFunc(myReshape); 
	glutIdleFunc(myIdle); 

	glEnable(GL_DEPTH);
	
  obstacles.resize(1);
  obstacles[0] = (Obstacle*) new PlaneObstacle(BfastVector3(0.0,0.0,1.0),0.0);

	glutMainLoop(); // start it running 
	return 0; // ANSI C expects this 
}

